#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import BackdropStyle as bs
from roller_constant_key import BackdropStyle as by, Option as ok
from roller_def import get_default_value
from roller_fu import (
    blur_selection,
    clone_layer,
    color_fill_layer,
    dilate,
    hide_layer,
    make_layer_group,
    merge_layer,
    merge_layer_group,
    select_rect
)
from roller_maya_style import Style, make_background
from roller_one_gegl import (
    edge, emboss, median_blur, unsharp_mask, waterpixels
)
from roller_view_real import (
    add_sub_base_group,
    add_wip_layer,
    do_gradient_for_layer,
    finish_style,
    insert_copy
)
from roller_view_hub import get_gradient_factors
import gimpfu as fu

pdb = fu.pdb


def make_style(v, maya):
    """
    Make a Backdrop Style layer.

    v: View
    maya: Style
    Return: layer or None
        style layer
    """
    j = v.j
    d = maya.value_d
    z = make_background(v, maya, d)
    parent = add_sub_base_group(v, maya, z=z)
    backdrop_z = clone_layer(z)
    group = make_layer_group(j, "WIP", parent=parent, z=z)
    z1 = clone_layer(z)

    blur_selection(z, 500)

    z1.name = "HSV Hue"
    z1.mode = fu.LAYER_MODE_HSV_HUE

    pdb.plug_in_gimpressionist(j, z1, "Painted_Rock")

    z = insert_copy(v, group,  z1)
    z.mode = fu.LAYER_MODE_DIFFERENCE

    edge(z)

    z = insert_copy(v, group, z)
    z.mode = fu.LAYER_MODE_EXCLUSION

    select_rect(j, *v.wip.rect)
    dilate(z)
    waterpixels(z)
    unsharp_mask(z, 3., 30., .0)

    z = insert_copy(v, group, z)
    z = clone_layer(z, n="Linear Light")
    z.mode = fu.LAYER_MODE_LINEAR_LIGHT

    blur_selection(z, 12)

    z = insert_copy(v, group, z)

    dilate(z)
    dilate(z)

    z.mode = fu.LAYER_MODE_GRAIN_EXTRACT
    z1 = merge_layer_group(group)
    group = make_layer_group(j, "WIP", parent=parent, z=z1)
    z2 = clone_layer(z1, n="Luma Lighten Only")

    waterpixels(z2)

    z2.mode = fu.LAYER_MODE_LUMA_LIGHTEN_ONLY
    z2 = merge_layer(z2)
    z3 = clone_layer(z2, n="Median Blur")

    median_blur(z3, 32, 50.)

    z4 = clone_layer(z3, n="Pin Light")
    z4.mode = fu.LAYER_MODE_PIN_LIGHT

    edge(z4)
    pdb.plug_in_erode(
        j, z4,
        1,                              # propagate black
        7,                              # RGB channels
        1.,                             # full rate
        7,                              # direction mask
        0,                              # low limit
        255                             # upper limit
    )

    z = insert_copy(v, group, z4)
    z2.mode = fu.LAYER_MODE_OVERLAY
    z.opacity = 33.

    pdb.plug_in_colortoalpha(j, z, (0, 0, 0))
    emboss(z2, v.glow_ball.azimuth, 12., 1.)
    hide_layer(z3)
    hide_layer(z4)

    e = get_default_value(by.GRADIENT_FILL)
    e[ok.GRADIENT] = d[ok.GRADIENT]
    e[ok.START_X], e[ok.END_X], e[ok.START_Y], e[ok.END_Y] = \
        get_gradient_factors(d[ok.GRADIENT_ANGLE])
    z1 = do_gradient_for_layer(v, e, group, len(group.layers) - 1)
    z = add_wip_layer(
        v, maya, "Back", group=group, offset=len(group.layers) + 1
    )
    z.opacity = z1.opacity = d[ok.GRADIENT_OPACITY]

    color_fill_layer(z, (64, 64, 64))
    blur_selection(backdrop_z, 500)
    pdb.gimp_image_reorder_item(j, backdrop_z, group, len(group.layers) + 1)
    return finish_style(merge_layer_group(parent), "Clay Chemistry")


class ClayChemistry(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.BBR

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        self.init_background(make_style, *q, **d)

    def do(self, v, d, is_change):
        """
        Produce output.

        v: View
        d: dict
            Backdrop Style Preset

        is_change: bool
            Is the state of the super Maya's matter and/or mask.
        """
        self.is_dependent = \
            d[ok.BBR][ok.BACKGROUND][ok.BACKDROP_TYPE] == bs.BACKDROP_IMAGE
        super(ClayChemistry, self).do(v, d, is_change)
