#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_key import BackdropStyle as by, Option as ok
from roller_def import get_default_value
from roller_maya_style import Style
from roller_fu import (
    clear_selection,
    clear_inverse_selection,
    clone_layer,
    flip_layer,
    merge_layer_group,
    select_polygon,
    select_rect
)
from roller_one import enumerate_name
from roller_one_rect_table import RectTable
from roller_one_the import The
from roller_view_real import (
    add_sub_base_group, clip_to_wip, do_gradient_for_layer, finish_style
)
from roller_view_hub import do_stylish_shadow
import gimpfu as fu

pdb = fu.pdb


def make_style(v, maya):
    """
    Do the Backdrop Style.

    v: View
        Has scope variable.

    maya: Style
    Return: layer
        with style material
    """
    def _do_gradient(_n):
        """
        Draw a gradient.

        _n: string
            name of the gradient layer

        Return: layer
            with gradient
        """
        _z = do_gradient_for_layer(v, e, group, len(group.layers) - 1)
        _z.name = _n
        return _z

    def _do_select():
        """Select an area defined by an array of points."""
        select_polygon(j, q, option=fu.CHANNEL_OP_ADD)

    def _double(_n, _m=0):
        """
        Duplicate a layer and flip it depending on a flag.

        _n: string
            name of flipped layer

        _m: flag
            If it's true, the layer is flipped horizontally.
        """
        _z = clone_layer(z, n=_n)

        flip_layer(_z, horizontal=_m)
        return _z

    j = v.j
    d = maya.value_d
    gradients = The.cat.gradient_list
    e = get_default_value(by.GRADIENT_FILL)
    e[ok.GRADIENT] = d[ok.GBR][ok.GRADIENT]
    left_x, top_y, w, h = v.wip.rect
    color = tuple([255 - i for i in d[ok.COLOR_1]])

    e.update(d)
    pdb.gimp_selection_none(j)

    grid = RectTable((0, 0, w, h), e[ok.ROW], e[ok.COLUMN]).table
    group = add_sub_base_group(v, maya)

    # Calculate the points using their relative position.
    y = h // 5 + top_y
    center_x = left_x + (w // 2)
    center_y = top_y + (h // 2)
    right_x = w + left_x
    bottom_y = h + top_y

    # The triangle is facing down.
    q = left_x, y, right_x, y, center_x, bottom_y

    e[ok.START_X], e[ok.START_Y] = (0, .5), (0, 1.)
    e[ok.END_X], e[ok.END_Y] = (0, .5), (0, .2)

    z = _do_gradient("Down Triangle")
    z.mode = fu.LAYER_MODE_DARKEN_ONLY

    _do_select()
    clear_inverse_selection(z)
    clip_to_wip(v, do_stylish_shadow(z, intensity=50.))

    pdb.gimp_selection_none(j)

    z1 = _double("Up Triangle")

    clip_to_wip(v, do_stylish_shadow(z1, intensity=50.))
    pdb.gimp_selection_none(j)

    # horizontal
    for r1 in range(0, e[ok.ROW], 2):
        select_rect(
            j,
            left_x,
            grid[r1][0].y + top_y,
            w,
            grid[r1][0].h,
            option=fu.CHANNEL_OP_ADD
        )

    clear_selection(z1)
    clip_to_wip(v, do_stylish_shadow(z1, intensity=50.))

    # row, 'a'
    a = grid[0]

    # vertical
    for c1 in range(0, e[ok.COLUMN], 2):
        select_rect(
            j, a[c1].x + left_x, top_y, a[c1].w, h, option=fu.CHANNEL_OP_ADD
        )

    clear_selection(z)
    do_stylish_shadow(z, intensity=50.)

    n = by.CORE_DESIGN

    # background gradient
    if n in gradients:
        n = enumerate_name(n, gradients)

    grad = e[ok.GRADIENT] = pdb.gimp_gradient_new(n)
    e[ok.OFFSET] = 0
    gradients += [grad]

    # segment index, '0'; end point opacity, '100.'
    pdb.gimp_gradient_segment_set_right_color(grad, 0, color, 100.)
    pdb.gimp_gradient_segment_set_left_color(grad, 0, d[ok.COLOR_1], 100.)

    # left
    e[ok.START_X] = 0, .0
    e[ok.START_Y] = e[ok.END_X] = e[ok.END_Y] = 0, .5

    pdb.gimp_selection_none(j)

    z = _do_gradient("Left")
    q = left_x, top_y, left_x, bottom_y, center_x, center_y

    _do_select()
    clear_inverse_selection(z)
    _double("Right", _m=1)

    # top
    e[ok.START_Y] = 0, .0
    e[ok.START_X] = e[ok.END_X] = e[ok.END_Y] = 0, .5
    z = _do_gradient("Top")
    q = left_x, top_y, right_x, top_y, center_x, center_y + 1

    _do_select()
    clear_inverse_selection(z)

    # bottom
    _double("Bottom")

    pdb.gimp_gradient_delete(grad)
    gradients.pop(-1)

    z = merge_layer_group(group)

    clip_to_wip(v, z)
    return finish_style(z, "Core Design")


class CoreDesign(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.GBR
    is_dependent = False

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        k_path = d['k_path']
        d['k_path'] = [k_path, k_path + (ok.GBR,), k_path + (ok.IRR,)]
        Style.__init__(self, *q + (make_style,), **d)
