#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_key import Option as ok
from roller_fu import add_layer, paste_layer, rotate_image
from roller_maya_style import Style
from roller_polygon import calc_circumradius
from roller_view_hub import fill_with_clipboard, make_cube_pattern
from roller_view_real import add_wip_layer, clip_to_wip, finish_style
import gimpfu as fu

pdb = fu.pdb


def make_style(v, maya):
    """
    Make a Backdrop Style layer.

    v: View
    maya: Style
    Return: layer
        with Cube Pattern
    """
    j = v.j
    d = maya.value_d
    key = maya.any_group.item.key
    make_cube_pattern(
        d[ok.HEIGHT_CLIP], d[ok.COLOR_3A], d[ok.IFRW][ok.FLIP_V]
    )

    if d[ok.ANGLE]:
        w = calc_circumradius(j.width, j.height) * 2
        j1 = pdb.gimp_image_new(w, w, fu.RGB)
        z = add_layer(j1, key)

        fill_with_clipboard(z)
        rotate_image(j1, d[ok.ANGLE])
        pdb.gimp_edit_copy_visible(j1)
        pdb.gimp_image_delete(j1)

        z = paste_layer(maya.group, n=key)

        pdb.gimp_image_reorder_item(j, z, maya.group, 0)
        pdb.gimp_layer_resize_to_image_size(z)
        clip_to_wip(v, z)

    else:
        # pattern layer, 'z'
        z = add_wip_layer(v, maya, key)
        fill_with_clipboard(z)

    if d[ok.IFRW][ok.INVERT]:
        pdb.gimp_drawable_invert(z, 0)
    return finish_style(z, "Cube Pattern")


class CubePattern(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.BRR
    is_dependent = False

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        k_path = d['k_path']
        d['k_path'] = [k_path, k_path + (ok.IFRW,)]
        Style.__init__(self, *q + (make_style,), **d)
