#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import BackdropStyle as bs
from roller_constant_key import Option as ok
from roller_fu import clone_layer, merge_layer, merge_layer_group
from roller_maya_style import Style, make_background
from roller_view_real import add_sub_base_group, finish_style
import gimpfu as fu

LAYER_NUM = "{} of {}"
pdb = fu.pdb


def make_style(v, maya):
    """
    Make a Backdrop Style layer.

    v: View
    maya: Rainbow Valley
    Return: layer
        with the style material
    """
    def _noise():
        """Adds noise to a layer."""
        if d[ok.POWER]:
            pdb.plug_in_solid_noise(
                j, z,
                0,                      # no tile-able
                1,                      # yes, turbulent
                seed_,
                7,                      # medium detail
                2,                      # horizontal size
                2                       # vertical size
            )
            pdb.plug_in_hsv_noise(
                j, z,
                4,                      # medium holdness
                d[ok.POWER],
                d[ok.POWER],
                d[ok.POWER]
            )

    j = v.j
    d = maya.value_d
    z = make_background(v, maya, d)
    parent = add_sub_base_group(v, maya, z=z)
    seed_ = d[ok.SEED] + v.glow_ball.seed

    # layer counter, 'x'
    x = 1

    # total layers to make, 'a'
    a = 15
    a += 8 if d[ok.TEXTURE] else 7

    for i in range(10):
        z = clone_layer(z, n=LAYER_NUM.format(x, a))
        z.mode = fu.LAYER_MODE_DIFFERENCE

        pdb.plug_in_despeckle(j, z, i, 0, 0, 255)

        if i % 2:
            x += 1
            z = clone_layer(z, n=LAYER_NUM.format(x, a))
            z.mode = fu.LAYER_MODE_LIGHTEN_ONLY

            _noise()

            seed_ += 1

            pdb.plug_in_despeckle(j, z, i, 0, 0, 255)
            z = merge_layer(z)

        is_engrave = False

        if d[ok.TEXTURE]:
            if i < 7 or i == 9:
                is_engrave = True

        elif i < 7:
            is_engrave = True

        if is_engrave:
            x += 1
            z = clone_layer(z, n=LAYER_NUM.format(x, a))

            pdb.plug_in_engrave(j, z, max(i, 2), 1)

            z.mode = fu.LAYER_MODE_BURN
            z = merge_layer(z)

            pdb.plug_in_wind(
                j, z,
                0,              # threshold
                2,              # from top
                50,             # strength
                1,              # blast
                1               # leading edge
            )
        x += 1
    return finish_style(merge_layer_group(parent), "Rainbow Valley")


class RainbowValley(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.BBR

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        self.init_background(make_style, *q, **d)

    def do(self, v, d, is_change):
        """
        Check, modify, and produce layer output.

        v: View
        d: dict
            Backdrop Style Preset

        is_change: bool
            Is the state of the super Maya's matter and/or mask.
        """
        self.is_dependent = \
            d[ok.BBR][ok.BACKGROUND][ok.BACKDROP_TYPE] == bs.BACKDROP_IMAGE
        super(RainbowValley, self).do(v, d, is_change)
