#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_key import Option as ok
from roller_fu import (
    clone_layer, color_fill_layer, merge_layer, merge_layer_group
)
from roller_maya_style import Style
from roller_view_real import (
    add_sub_base_group, add_wip_layer, clip_to_wip, finish_style
)
import gimpfu as fu

EDGE = "{} of {}"
pdb = fu.pdb


def do_edge(z, mode, i, reps):
    """
    Shred the layer with the edge function and a layer mode.

    z: layer
        work-in-progress
        Is modified.

    mode: enum
        layer mode

    i: int
        repetition number

    reps: int
        number of repetitions

    Return: layer
        with modifications
    """
    j = z.image

    pdb.plug_in_edge(j, z, 1., 0, 0)

    z.mode = mode
    z = merge_layer(z)
    return clone_layer(z, n=EDGE.format(str(i), str(reps)))


def make_style(v, maya):
    """
    Make a Backdrop Style layer.

    v: View
    maya: RockyLanding
    Return: layer
        with the style material
    """
    j = v.j
    d = maya.value_d
    z = add_wip_layer(v, maya, "Rocky Landing WIP")
    reps = d[ok.BLEND]
    reps1 = reps * 2 + 4
    group = add_sub_base_group(v, maya, z=z)

    pdb.gimp_selection_none(j)
    pdb.plug_in_plasma(j, z, d[ok.SEED], 3)

    z1 = clone_layer(z, n=EDGE.format("1", reps1))

    color_fill_layer(z, (0, 0, 0))

    z = clone_layer(z1, n=EDGE.format("2", reps1))
    z = do_edge(z, fu.LAYER_MODE_SUBTRACT, 3, reps1)
    z = do_edge(z, fu.LAYER_MODE_LCH_HUE, 4, reps1)
    x = 5

    for i in range(reps):
        z = do_edge(z, fu.LAYER_MODE_COLOR_ERASE, x, reps1)
        z = do_edge(z, fu.LAYER_MODE_LCH_CHROMA, x + 1, reps1)
        x += 2

    z.mode = fu.LAYER_MODE_GRAIN_EXTRACT
    z1 = clone_layer(z, n="Overlay")

    pdb.plug_in_emboss(j, z1, v.glow_ball.azimuth, 30., 1, 1)

    z1.mode = fu.LAYER_MODE_OVERLAY

    pdb.gimp_drawable_invert(z1, 0)

    z = clone_layer(z1, n="Despeckle")

    pdb.plug_in_despeckle(j, z1, 1, 1, 200, 255)
    pdb.gimp_layer_set_offsets(z, v.wip.x, v.wip.y + 2)
    pdb.plug_in_sobel(j, z, 1, 0, 0)

    z.opacity = 83.
    z1.mode = fu.LAYER_MODE_NORMAL
    z1.opacity = 33.

    pdb.gimp_image_reorder_item(j, z1, group, 3)

    z = merge_layer_group(group)

    clip_to_wip(v, z)
    return finish_style(z, "Rocky Landing")


class RockyLanding(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.BRR
    is_dependent = False

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        Style.__init__(self, *q + (make_style,), **d)
