#!/usr/bin/env python
# -*- coding: utf-8 -*-
from random import randint, uniform
from roller_constant_for import BackdropStyle as bs
from roller_constant_key import Option as ok
from roller_fu import blur_selection, merge_layer_group
from roller_one import random_rgb
from roller_maya_style import Style, make_background
from roller_view_real import finish_style
from roller_view_hub import set_brush_details
from roller_view_preset import combine_seed
from roller_view_real import add_sub_base_group, add_wip_layer, clip_to_wip
from roller_view_shadow import make_shadow
import gimpfu as fu
import math

pdb = fu.pdb


def make_style(v, maya):
    """
    Make a Backdrop Style layer.

    v: View
    maya: TrailingVine
    Return: layer or None
        with the style material
    """
    def _plot_wave():
        """
        Calculate the points for a wave that starts
        at the top of the layer and ends at its bottom.

        Return: list
            of x and y point
            to plot
        """
        _q = []
        _y = _angle = .0

        # Pi x 2, '6.28318'
        _f = 6.28318 / wave_length

        while _y < bottom_y:
            _x = math.sin(_angle) * amplitude + x
            _q += [_x, _y]
            _y += 1
            _angle += _f
        return _q

    j = v.j
    d = maya.value_d
    z = make_background(v, maya, d)
    group = add_sub_base_group(v, maya, z=z)

    e = {
        ok.SHADOW_BLUR: 20,
        ok.INTENSITY: 100,
        ok.OFFSET_X: -6,
        ok.OFFSET_Y: 0,
        ok.SHADOW_COLOR: (0, 0, 0)
    }
    w, h = v.wip.size
    span = min(w, h)
    count = d[ok.LAYER_COUNT]
    shadow_blur_dec = 90. / count
    bottom_y = h + v.wip.y

    # Preserve.
    foreground = pdb.gimp_context_get_foreground()

    combine_seed(v, d)
    set_brush_details()

    for i in range(count):
        amplitude = max(20, w // max(w, randint(span // 33, span // 11)))
        amp_inc = max(10, w // randint(50, 100))
        offset = h * uniform(-.5, .5)
        wave_length = h + offset
        z = add_wip_layer(
            v, maya, "{} of {}".format(i + 1, count), group=group
        )
        z.mode = fu.LAYER_MODE_DIFFERENCE
        e[ok.SHADOW_BLUR] = max(30, (90 - i * 10 - shadow_blur_dec))

        # pixel size, '1.5'
        pdb.gimp_context_set_brush_size(1.5 + i // 2)
        pdb.gimp_context_set_foreground(random_rgb())

        for _ in range(int(d[ok.WAVE_PER_LAYER])):
            x = randint(0, w)
            q = _plot_wave()

            pdb.gimp_selection_none(j)
            pdb.gimp_paintbrush_default(z, len(q), q)
            amplitude += amp_inc

        z2 = make_shadow(j, e, group, (z,))

        pdb.gimp_image_reorder_item(j, z2, group, 1)
        blur_selection(z, (count - i) * 3)

    z = merge_layer_group(group)

    # Restore.
    pdb.gimp_context_set_foreground(foreground)

    clip_to_wip(v, z)
    return finish_style(z, "Trailing Vine")


class TrailingVine(Style):
    """Create Backdrop Style output."""
    bump_row_k = ok.BBR

    def __init__(self, *q, **d):
        """
        q: tuple
            Style spec

        d: dict
            Style spec
        """
        self.init_background(make_style, *q, **d)

    def do(self, v, d, is_change):
        """
        Check, modify, and produce layer output.

        v: View
        d: dict
            Backdrop Style Preset

        is_change: bool
            Is the state of the super Maya's matter and/or mask.
        """
        self.is_dependent = \
            d[ok.BBR][ok.BACKGROUND][ok.BACKDROP_TYPE] == bs.BACKDROP_IMAGE
        super(TrailingVine, self).do(v, d, is_change)
