#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_key import Option as ok
from roller_frame import Metal, do_soft_metal_material
from roller_fu import (
    add_layer,
    clear_selection,
    color_fill_layer,
    copy_all_image,
    select_ellipse
)
from roller_view_hub import fill_with_clipboard
import gimpfu as fu

pdb = fu.pdb


def make_pattern(z, d):
    """
    Draw circles on a layer. Create a pattern (a black circle). Use the
    clipboard to hold the pattern. Fill the provided layer with the pattern.

    z: layer
        to receive pattern

    d: dict
        Circle Punch Preset
        {Option key: value}

    Return: layer
        work-in-progress
        Has material to select.
    """
    diameter = d[ok.CIRCLE_DIAMETER]
    w = int(diameter * 1.5)
    h = int(w * .8660254)
    j = pdb.gimp_image_new(w, h, fu.RGB)
    z1 = add_layer(j, 'pattern')

    # A circle at the center of 'z1' will become a hole in the pattern.
    x = (w - diameter) // 2
    y = (h - diameter) // 2

    color_fill_layer(z1, (255, 255, 255))
    select_ellipse(j, x, y, diameter, diameter)
    pdb.gimp_selection_feather(j, 1.)
    clear_selection(z1)

    # Set the Clipboard Image.
    copy_all_image(j)

    pdb.gimp_image_delete(j)
    pdb.gimp_selection_none(z.image)
    fill_with_clipboard(z)
    return z


def do_matter(v, maya):
    """
    Make a frame.

    v: View
    maya: Frame
    Return: layer
        with the frame
    """
    return do_soft_metal_material(v, maya, make_pattern, "Circle Punch")


class CirclePunch(Metal):
    """Has circular holes in a metallic frame."""

    def __init__(self, *q, **d):
        """
        q: tuple
            Metal spec

        d: dict
            Metal spec
        """
        Metal.__init__(self, *q + (do_matter,), **d)
