#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Mask as ms
from roller_constant_key import Option as ok
from roller_maya import Maya, check_matter
import gimpfu as fu

pdb = fu.pdb


def do_mask_matter(v, maya):
    """
    Produce mask output.

    v: View
    maya: Work
    Return: layer or None
        with mask
    """
    return maya.do_mask(v, maya.super_maya)


class Mask(Maya):
    """Manage Mask Preset layer output."""
    put = (check_matter, 'matter'),
    is_seeded = True
    issue_q = 'matter',

    def __init__(
        self, any_group, super_maya, view_x, do_mask, get_mask_d, k_path
    ):
        """
        any_group: AnyGroup
        super_maya: Maya
            Is from the enclosing Preset.

        view_x: int
            Path or Work
            0 or 1

        do_mask: function
            Callback to do produce a Mask layer.

        get_mask_d: function
            Callback to get the Mask's value dict.

        k_path: tuple
            (Option key,)
            Is where the vote is kept in the AnyGroup vote dict.
        """
        self.super_maya = super_maya
        self.vote_type = super_maya.vote_type
        self.do_mask = do_mask
        self.do_matter = do_mask_matter
        self.get_mask_d = get_mask_d

        Maya.__init__(
            self,
            any_group,
            view_x,
            Mask.put,
            k_path=[
                k_path,
                k_path + (ok.IBR, ok.IMAGE_CHOICE,),
                k_path + (ok.IBR, ok.BRUSH_D,)
            ]
        )
        self.set_issue()

    def do(self, v, d):
        """
        Manage layer output during a View run.

        v: View
        d: dict
            Mask Preset
            {Option key: value}
        """
        self.value_d = d
        self.go = d[ok.SWITCH] and self.super_maya.go

        if self.go:
            self.is_matter |= self.super_maya.is_matter
        self.realize_vote(v)

    def on_global_seed(self, _, arg):
        """
        The Global Random Seed option has changed.
        Check to see if the Mask output is seed dependent.

        _: Power
            Sent the signal.

        arg: tuple
            (Plan vote, Work vote)
        """
        d = self.get_mask_d()

        if d and d[ok.MASK_TYPE] == ms.FRINGE:
            super(Mask, self).on_global_seed(_, arg)
        else:
            self.great_vote_d = {}
