#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Shape as sh
from roller_model_goo import Goo
from roller_one_rect import Rect
from roller_polygon import calc_pin_xy

RATIO = sh.TRIANGLE_SCALE_RATIO_DOWN
RATIO_ALT = 1 - RATIO


class GridCircle:
    """
    Calculate the position and the size of cells. Cells are
    ellipse shaped. Is a double-spaced cell model-type.
    """

    @staticmethod
    def calc(model, o):
        """
        Calculate cell rectangle and form polygon for a Model's cell.

        model: Model
        o: One
            with Cell Type reference

        Return: list
            [Plan vote, Work vote]
        """
        vote_d = {}
        did_cell = model.past.did_cell
        row, column = model.division
        goo_d = model.goo_d
        shape = model.cell_shape
        x, y, canvas_w, canvas_h = model.canvas_pocket.rect

        if shape == sh.CIRCLE_HORIZONTAL:
            # cell size
            w1 = canvas_w / (.5 + column * .5)
            h1 = canvas_h / (RATIO_ALT + row * RATIO)
            w = min(w1, h1)
            w, h = w, w

            # grid size
            w1 = w / 2.
            h1 = h * RATIO
            h2 = h * RATIO_ALT
            s = column * w1 + w1, row * h1 + h2
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

            # intersect
            w1 = w / 2.
            h1 = h * RATIO

        else:
            # circle, vertically aligned
            # cell size
            w1 = canvas_w / (RATIO_ALT + column * RATIO)
            h1 = canvas_h / (.5 + row * .5)
            w = min(w1, h1)
            w, h = w, w

            # grid size
            w1 = w * RATIO
            w2 = w * RATIO_ALT
            h1 = h / 2.
            s = column * w1 + w2, row * h1 + h1
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

            # intersect
            w1 = w * RATIO
            h1 = h / 2.

        # [intersect point]
        q_x = []
        q_y = []

        # remainder total, 'f_x, f_y'
        f_x = f_y = .0

        # On remainder full, add one to the coordinate.
        for c in range(column):
            x, f = divmod(x, 1.)
            f_x += f

            if f_x >= .999:
                x += 1.
                f_x -= 1.

            q_x.append(x)
            x += w1

        for r in range(row):
            y, f = divmod(y, 1.)
            f_y += f

            if f_y >= .999:
                y += 1.
                f_y -= 1.

            q_y.append(y)
            y += h1

        # Prevent round to zero with max 1.
        w = max(1., w)

        for r_c in model.cell_q:
            r, c = r_c
            y = q_y[r]
            x = q_x[c]
            a = goo_d[r_c] = Goo(r_c)
            a.cell = a.merged = Rect(x, y, w, w)
            vote_d[r_c] = did_cell(r_c)
            a.form = x, y, w, w
        return vote_d
