#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Grid as gr, Shape as sh
from roller_model_goo import Goo
from roller_one_rect import Rect
from roller_polygon import (
    arrange_octagon,
    arrange_octagon_side_to_side,
    calc_octagon_offset,
    calc_octagon_side_to_side_offset,
    calc_pin_xy
)


class GridOctagon:
    """Calculate the coordinates and the size of octagon shaped cells."""

    @staticmethod
    def calc(model, o):
        """
        Calculate cell rectangle for Model cell.

        model: Model
        o: One
            with Cell Type reference

        Return: list
            [Plan vote, Work vote]
        """
        vote_d = {}
        did_cell = model.past.did_cell
        row, column = model.division
        goo_d = model.goo_d
        x, y = model.canvas_pocket.position
        canvas_w, canvas_h = map(float, model.canvas_pocket.size)
        is_align = False if model.cell_shape in \
            (sh.OCTAGON_SHEAR, sh.OCTAGON) else True

        # intersect points
        q_x = []
        q_y = []

        if o.grid_type == gr.CELL_SIZE:
            # Correct cell size overflow.
            w, h = min(canvas_w, o.column_width), min(canvas_h, o.row_height)

            s = w * column, h * row
            w, h = w / 1., h / 1.
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

        elif o.grid_type == gr.SHAPE_COUNT:
            w = canvas_w / column
            h = canvas_h / row
            w = min(w, h)
            s = w * column, w * row
            w, h = w, w
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

        else:
            w = canvas_w / column
            h = canvas_h / row

        # remainder total, 'f_x, f_y'
        f_x = f_y = .0

        # Calculate cell rectangle.
        # Add remainder to coordinate when the remainder value is close to one.
        for _ in range(column + 1):
            x, f = divmod(x, 1.)
            f_x += f

            if f_x >= .999:
                x += 1.
                f_x -= 1.

            q_x.append(x)
            x += w

        for _ in range(row + 1):
            y, f = divmod(y, 1.)
            f_y += f

            if f_y >= .999:
                y += 1.
                f_y -= 1.

            q_y.append(y)
            y += h

        offset_q = calc_octagon_side_to_side_offset(w, h) if is_align \
            else calc_octagon_offset(w, h)

        for r_c in model.cell_q:
            r, c = r_c
            x, x1 = q_x[c], q_x[c + 1]
            y, y1 = q_y[r], q_y[r + 1]
            a = goo_d[r_c] = Goo(r_c)

            # Prevent round to zero with max 1.
            w, h = max(1., x1 - x), max(1., y1 - y)
            a.cell = a.merged = Rect(x, y, w, h)
            a.form = arrange_octagon_side_to_side(offset_q, x, y, w, h) \
                if is_align else arrange_octagon(offset_q, x, y, w, h)
            vote_d[r_c] = did_cell(r_c)
        return vote_d
