#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Grid as gr
from roller_model_goo import Goo
from roller_one_rect import Rect
from roller_polygon import calc_pin_xy


class GridRhombus:
    """
    Calculate the coordinates and the size of cells.
    The cells are Rhombus shaped.
    """

    @staticmethod
    def calc(model, o):
        """
        Calculate cell rectangle and form polygon for Model cell.

        model: Model
        o: One
            with Cell Type reference

        Return: list
            [Plan vote, Work vote]
        """
        vote_d = {}
        did_cell = model.past.did_cell
        row, column = model.division
        goo_d = model.goo_d
        x, y, canvas_w, canvas_h = model.canvas_pocket.rect

        # intersect points
        q_x, q_y = [], []

        if o.grid_type == gr.CELL_SIZE:
            # Correct cell size overflow.
            w, h = min(canvas_w, o.column_width), min(canvas_h, o.row_height)

            # grid size
            w1, h1 = w / 2., h / 2.
            s = w + (column - 1) * w1, h + (row - 1) * h1
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

        elif o.grid_type == gr.SHAPE_COUNT:
            # cell size
            w = canvas_w / (.5 + column * .5)
            h = canvas_h / (.5 + row * .5)
            w = min(w, h) / 1.
            w, h = w, w

            # grid size
            h1 = h / 2.
            s = column * h1 + h1, row * h1 + h1
            x, y = calc_pin_xy(o.pin_corner, x, y, canvas_w, canvas_h, *s)

        else:
            w = canvas_w / (.5 + column * .5)
            h = canvas_h / (.5 + row * .5)

        # intersects
        w1 = w / 2.
        h1 = h / 2.

        # remainder total, 'f_x, f_y'
        f_x = f_y = .0

        # Calculate cell rectangle.
        # Increment the coordinate when the remainder is close to one.
        for c in range(column + 2):
            x, f = divmod(x, 1.)
            f_x += f

            if f_x >= .999:
                x += 1.
                f_x -= 1.

            q_x.append(x)
            x += w1

        for r in range(row + 2):
            y, f = divmod(y, 1.)
            f_y += f

            if f_y >= .999:
                y += 1.
                f_y -= 1.

            q_y.append(y)
            y += h1

        for r_c in model.cell_q:
            r, c = r_c
            y, y1, y2 = q_y[r], q_y[r + 1], q_y[r + 2]
            x, x1, x2 = q_x[c], q_x[c + 1], q_x[c + 2]
            a = goo_d[r_c] = Goo(r_c)

            # Prevent round to zero with max 1.
            a.cell = a.merged = Rect(x, y, max(1., x2 - x), max(1., y2 - y))

            a.form = x, y1, x1, y, x2, y1, x1, y2
            vote_d[r_c] = did_cell(r_c)
        return vote_d
